import toml
import click
from pathlib import Path

from svtter_template_creator import __version__, lib, repl as r, utils as u


# create a config file.
u.create_config()


@click.group()
def cli():
    pass


@click.command()
@click.option(
    "--name",
    prompt="template name",
    help="The template need to create",
    type=click.Choice(lib.get_choice()),
)
def create(name):
    """
    create template via name
    """
    lib.create(name)


@click.command()
def version():
    """show version information"""
    click.echo(f"ttc version is: {__version__}")


@click.command(help="start a repl environment")
def repl():
    click.echo(f"ttc version is: {__version__}")
    r.repl()


@click.command(help="write version to __init__.py and pyproject.toml")
@click.option("--version", help="The new verison!", required=True)
def write(version):
    filepath = "src/ttc/__init__.py"
    with open(filepath, "w") as f:
        f.write(f'__version__ = "{version}"')

    p = Path("./pyproject.toml")
    res = toml.load(p)
    res["tool"]["poetry"]["version"] = version
    # write back
    with open(p, "w") as f:
        toml.dump(res, f)


@click.command(help="show the config file")
def config():
    click.echo("-----------------------------")
    click.echo("file path:")
    click.echo(u.get_config().path)
    click.echo("-----------------------------")
    click.echo("config content:")
    click.echo("-----------------------------")
    click.echo(u.get_config().content)


enabled_cmd = [
    create,
    version,
    write,
    repl,
    config,
]

for c in enabled_cmd:
    cli.add_command(c)



import sys
import prompt_toolkit as pt
import pathlib as p
from prompt_toolkit import completion as cmp
from prompt_toolkit import history as hst
from prompt_toolkit import auto_suggest
from svtter_template_creator import lib


cmd_list = ["help", "create"]


class SimpleCompleter(cmp.Completer):
    def get_completions(self, document, complete_event):
        word_before_cursor = document.get_word_before_cursor()
        for command in cmd_list:
            if command.startswith(word_before_cursor):
                yield cmp.Completion(command, -len(word_before_cursor))


def repl():
    session = pt.PromptSession(
        history=hst.FileHistory(str(p.Path.home() / ".config" / "tt" / "history.txt")),
        completer=SimpleCompleter(),
        auto_suggest=auto_suggest.AutoSuggestFromHistory(),
    )

    while True:
        try:
            cmd_str = session.prompt("tt> ")
            if cmd_str.strip() == "exit":
                break
            else:
                cmd, args = cmd_str.split(" ")
                if cmd not in cmd_list:
                    print(f"error cmd: {cmd}")
                else:
                    lib.create(args)
        except EOFError:
            return
        except KeyboardInterrupt:
            continue
        except Exception as e:
            print(e, file=sys.stderr)



import pathlib as p
import typing as t
from collections import namedtuple

default = """[repos]
compose='git@github.com:svtter/cookiecutter-compose.git'
django='git@github.com:svtter/cookiecutter-django.git'
package='git@github.com:svtter/cookiecutter-pypackage.git'
"""


def create_config():
    """
    create a config folder
    """
    p.Path.home().joinpath(".config", "tt").mkdir(parents=True, exist_ok=True)
    tt_file = p.Path.home().joinpath(".config", "tt.toml")

    if tt_file.exists():
        return
    tt_file.write_text(default)


Config = namedtuple("Config", ["path", "content"])


def get_config() -> Config:
    p.Path.home().joinpath(".config", "tt").mkdir(parents=True, exist_ok=True)
    tt_file = p.Path.home().joinpath(".config", "tt.toml")

    return Config(str(tt_file), tt_file.read_text())



import typing as t
from warnings import warn
import tomli
import pathlib
import os


def get_dict():
    warn("get_dict is deprecated, use read_repo instead")

    prefix = os.getenv("TC_URL", "git@github.com:svtter")

    template_dict = {
        "django": "{prefix}/cookiecutter-django.git".format(prefix=prefix),
        "package": "{prefix}/cookiecutter-pypackage.git".format(prefix=prefix),
        "compose": "{prefix}/cookiecutter-compose.git".format(prefix=prefix),
    }
    return template_dict


def create(name) -> None:
    """
    create template via name
    """
    # repo: dict[name, url]
    template_dict = read_repo()

    template = template_dict.get(name, None)
    if template is None:
        raise ValueError(f"template {name} not found")
    os.system(f"cookiecutter {template}")


def get_choice() -> t.List[str]:
    """get the choice from toml file"""
    repos = read_repo()
    return list(repos.keys())


def read_repo() -> t.Dict[str, str]:
    """get the repos from toml file"""
    c_path = pathlib.Path.home() / ".config" / "tt.toml"
    with open(c_path, "rb") as fp:
        config = tomli.load(fp)

    return config["repos"]
